@@ -64,6 +64,10 @@ and
 
 type protocol = Icy | Http
 
+let string_of_protocol = function
+  | Icy  -> "icy"
+  | Http -> "http"
+
 type content_type = string
 
 let ogg_application = "application/ogg"
@@ -85,6 +89,26 @@ type connection =
     headers      : (string,string) Hashtbl.t
   }
 
+let string_of_connection c = 
+  Printf.sprintf 
+    "{ \"mount\": %S,\n\
+       \"user\":  %S,\n\
+       \"password\": %S,\n\
+       \"host\": %S,\n\
+       \"port\": %d,\n\
+       \"content_type\": %S,\n\
+       \"protocol\": %S,\n\
+       \"headers\": { %s } }"
+     c.mount
+     c.user
+     c.password
+     c.host
+     c.port
+     (string_of_content_type c.content_type)
+     (string_of_protocol c.protocol)
+     (Hashtbl.fold (fun x y z -> Printf.sprintf "%S: %S,\n%s" x y z)
+                   c.headers "")
+
 type audio_info = (string,string) Hashtbl.t 
 
 type metadata = (string,string) Hashtbl.t
@@ -100,11 +124,12 @@ type status = Connected of connection_data | Disconnected
 
 type t = 
   { 
-    ipv6            : bool;
-    timeout         : float;
-    bind            : string option;
-    mutable icy_cap : bool;
-    mutable status  : status_priv
+    ipv6               : bool;
+    timeout            : float;
+    connection_timeout : float option;
+    bind               : string option;
+    mutable icy_cap    : bool;
+    mutable status     : status_priv
   }
 
 let get_connection_data x =
@@ -141,13 +166,14 @@ let create_socket ?(ipv6=false) ?bind () =
        end;
        raise (Error (Create e))
 
-let create ?(ipv6=false) ?bind ?(timeout=30.) () = 
+let create ?(ipv6=false) ?bind ?connection_timeout ?(timeout=30.) () = 
   { 
-    ipv6    = ipv6;
-    timeout = timeout;
-    bind    = bind;
-    icy_cap = false;
-    status  = PrivDisconnected
+    ipv6               = ipv6;
+    timeout            = timeout;
+    connection_timeout = connection_timeout;
+    bind               = bind;
+    icy_cap            = false;
+    status             = PrivDisconnected
   }
 
 let close x =
@@ -509,12 +535,30 @@ let connect_icy c socket source =
        raise e
 
 (** This function does *not* close the socket in case of error.. *)   
-let connect_socket socket host port = 
+let connect_socket ?timeout socket host port = 
+  let do_timeout = timeout <> None in
+  let check_timeout () =
+    match timeout with
+      | Some timeout ->
+         (* Block in a select call for [timeout] seconds. *)
+         let _, w, _ = Unix.select [] [socket] [] timeout in
+         if w = [] then
+           raise (Error (Connect Timeout)) ;
+         Unix.clear_nonblock socket
+       | None -> assert false
+  in
   try
+    if do_timeout then
+      Unix.set_nonblock socket ;
     Unix.connect 
       socket 
-      (Unix.ADDR_INET((Unix.gethostbyname host).Unix.h_addr_list.(0),port))
+      (Unix.ADDR_INET((Unix.gethostbyname host).Unix.h_addr_list.(0),port)) ;
+    if do_timeout then
+      Unix.clear_nonblock socket
   with
+    | Unix.Unix_error(Unix.EINPROGRESS, _, _) -> check_timeout ()
+    | Unix.Unix_error(Unix.EWOULDBLOCK, _, _) when Sys.os_type = "Win32" ->
+        check_timeout ()
     | e -> raise (Error (Connect e))
  
 let connect c source =
@@ -527,7 +571,7 @@ let connect c source =
                              ?bind:c.bind ()
   in
   try
-    connect_socket socket source.host port;
+    connect_socket ?timeout:c.connection_timeout socket source.host port;
     (* We do not know icy capabilities so far.. *)
     c.icy_cap <- false;
     match source.protocol with
@@ -545,7 +589,7 @@ let connect c source =
 
 let http_meta_request = 
   Printf.sprintf 
-    "GET /admin/metadata?mode=updinfo&mount=%s%s HTTP/1.0\r\n%s\r\n"
+    "GET /admin/metadata?mode=updinfo&mount=%s%s%s HTTP/1.0\r\n%s\r\n"
 
 let icy_meta_request = 
   Printf.sprintf 
@@ -553,8 +597,8 @@ let icy_meta_request =
 
 let manual_update_metadata 
        ~host ~port ~protocol ~user ~password 
-       ~mount ?(timeout=30.) ?headers ?(ipv6=false)
-       ?bind m =
+       ~mount ?(connection_timeout=5.) ?(timeout=30.) ?headers ?(ipv6=false)
+       ?bind ?charset m =
   let mount = 
     if mount.[0] <> '/' then
       "/" ^ mount
@@ -574,7 +618,7 @@ let manual_update_metadata
       | e -> raise (Error (Close e))
   in
   try
-    connect_socket socket host port;
+    connect_socket ~timeout:connection_timeout socket host port;
     let user_agent =
       try
         Hashtbl.find headers "User-Agent"
@@ -588,6 +632,11 @@ let manual_update_metadata
       else
         ""
     in
+    let charset = 
+      match charset with
+        | Some c -> Printf.sprintf "&charset=%s" c
+        | None   -> ""
+    in
     let user_agent = 
       Printf.sprintf "User-Agent: %s%s\r\n" user_agent agent_complement
     in
@@ -606,7 +655,7 @@ let manual_update_metadata
             let headers = 
               Printf.sprintf "Authorization: %s\r\n%s" (get_auth user password) user_agent
             in
-            http_meta_request mount meta headers
+            http_meta_request mount charset meta headers
         | Icy -> icy_meta_request password meta user_agent
     in
     write_data ~timeout socket request;
@@ -626,7 +675,7 @@ let manual_update_metadata
        end ;
        raise e
 
-let update_metadata c m = 
+let update_metadata ?charset c m = 
   if not c.icy_cap then                                                     
     raise (Error Invalid_usage);
   let data = get_connection_data c in
@@ -639,11 +688,12 @@ let update_metadata c m =
   let mount = source.mount in
   let host = source.host in
   let timeout = c.timeout in
+  let connection_timeout = c.connection_timeout in
   manual_update_metadata 
        ~host ~port ~protocol 
        ~user ~password ~timeout
-       ~mount ?headers
-       m
+       ~mount ?headers ?connection_timeout
+       ?charset m
 
 let send c x =
   let d = get_connection_data c in
