@@ -59,6 +59,9 @@ val string_of_error : exn -> string
   * provided you set the correct content-type (mime) for the source. *)
 type protocol = Icy | Http
 
+(** Return a string representation of a protocol. *)
+val string_of_protocol : protocol -> string
+
 (** Special type for content-type (mime) data. *)
 type content_type
 
@@ -96,6 +99,9 @@ type connection =
     headers      : (string, string) Hashtbl.t
   }
 
+(** Returns a JSON string representation of a connection. *)
+val string_of_connection : connection -> string
+
 (** Type for audio informations. Used for connection headers.
   * See [audio_info] function for more details. *)
 type audio_info = (string, string) Hashtbl.t
@@ -123,7 +129,8 @@ type t
   * [timeout] is [30.] by default. *)
 val create :
   ?ipv6:bool ->
-  ?bind:string -> 
+  ?bind:string ->
+  ?connection_timeout:float -> 
   ?timeout:float -> unit -> t
 
 (** Get a handler's status *)
@@ -199,7 +206,7 @@ val connect : t -> connection -> unit
   *
   * Raises: [Error Not_connected]
   * if not connected. *)
-val update_metadata : t -> metadata -> unit
+val update_metadata : ?charset:string -> t -> metadata -> unit
 
 (** Manually update metadata on any source without necessarily
   * being connected to it for streaming.
@@ -214,10 +221,12 @@ val manual_update_metadata :
            user:string ->
            password:string ->
            mount:string ->
+           ?connection_timeout:float ->
            ?timeout:float ->
            ?headers:(string, string) Hashtbl.t ->
            ?ipv6:bool -> ?bind:string ->
-           metadata -> unit
+           ?charset:string -> metadata -> 
+           unit
 
 (** Send data to a source connection. 
   *
